angular.module("umbraco.directives")
    .directive('umbFormsDesigner', function (dialogService, formService) {
        return {
            scope: {
                form: "=",
                fieldtypes: "=",
                prevaluesources: "="
            },

            transclude: true,

            restrict: 'E',
            replace: true,

            templateUrl: '/App_Plugins/UmbracoForms/directives/umb-forms-designer.html',
            link: function (scope, element, attrs, ctrl) {


                // *********************************************
                // Form management functions
                // *********************************************
                scope.initForm = function(form, fieldtypes){
                    formService.initForm(form, fieldtypes);
                    scope.gotoPageIndex(0);
                };


                // *********************************************
                // Page management functions
                // *********************************************

                scope.gotoPageIndex = function(index){
                    scope.form.$currentPage = scope.form.pages[index];
                    scope.currentPageIndex = index;

                    scope.onFirstPage = scope.currentPageIndex === 0;
                    scope.onLastPage = scope.currentPageIndex === scope.form.pages.length-1;
                };

                scope.addPage = function (form) {
                    scope.closeItemOverlay();
                    formService.addPage(form);
                    scope.gotoPageIndex(form.pages.length -1);

                };

                scope.deletePage = function (page) {
                    var index = scope.form.pages.indexOf(page);
                    scope.form.pages.splice(index, 1);
                    if (index > 0) {
                        scope.currentPageIndex = index - 1;
                        scope.form.$currentPage = scope.form.pages[index - 1];
                    } else {
                        scope.currentPageIndex = 0;
                        scope.form.$currentPage = scope.form.pages[0];
                    }

                    scope.onFirstPage = scope.currentPageIndex === 0;
                    scope.onLastPage = scope.currentPageIndex === scope.form.pages.length - 1;

                    populateFields();
                };

                // *********************************************
                // Fieldset management functions
                // *********************************************

                scope.setCurrentFieldset = function (fieldset) {
                    scope.currentFieldset = fieldset;
                };

                scope.addFieldset = function(page, fieldset, container, index){
                    scope.closeItemOverlay();
                    var _index = page.fieldSets.indexOf(fieldset);
                    formService.addFieldset(page, _index+1);
                };

                scope.deleteFieldset = function(page,fieldset) {
                    formService.deleteFieldset(page, fieldset);
                };

                scope.editFieldset = function (fieldset) {
                    populateFields();
                    dialogService.open(
                    {
                        template: "/app_plugins/UmbracoForms/Backoffice/Form/dialogs/fieldsetsettings.html",
                        fieldset: fieldset,
                        fields: scope.fields
                    });
                };

                // *********************************************
                // Container management functions
                // *********************************************
                scope.addContainer = function (fieldset, container, index) {
                    scope.closeItemOverlay();
                    var _index = fieldset.containers.indexOf(container);
                    formService.splitContainer(fieldset, container, _index+1);
                };

                // *********************************************
                // Field management functions
                // *********************************************
                scope.initField = function(field){
                    if(field && !field.$fieldType){
                        formService.setFieldType(field,field.fieldTypeId);
                    }
                };

                scope.addField = function (container, fieldtype, index) {
                    scope.closeItemOverlay();
                    formService.addField(container, fieldtype, index+1);
                };

                scope.editField = function(field){
                    populateFields();

                    scope.setFieldType(field);

                    dialogService.open(
                            {
                                template: "/app_plugins/UmbracoForms/Backoffice/Form/dialogs/fieldsettings.html",
                                field: field,
                                fields: scope.fields,
                                setFieldType: formService.setFieldType,
                                fieldtypes: scope.fieldtypes,
                                prevaluesources: scope.prevaluesources
                            });
                };

                scope.deleteField = function(fieldset,container,field) {
                    formService.deleteField(fieldset,container,field);
                };

                scope.copyField = function (container, field) {
                    var copy ={};
                    angular.copy(field, copy);
                    copy.id = generateGUID();
                    copy.caption = "copy of " + copy.caption;
                    container.fields.splice(container.fields.indexOf(field) + 1, 0, copy);
                    populateFields();
                };

                scope.setFieldType = function (field) {

                    //set settings
                    angular.forEach(field.settings, function (setting) {
                        for (var key in field.settings) {
                            if (field.settings.hasOwnProperty(key)) {
                                if (_.where(field.$fieldType.settings, { alias: key }).length > 0) {
                                    _.where(field.$fieldType.settings, { alias: key })[0].value = field.settings[key];
                                }
                            }
                        }
                    });

                };


                // *********************************************
                // Button/hover state handlers
                // *********************************************
                scope.setCurrentControl = function (field) {
                    scope.currentControl = field;
                };

                scope.setCurrentToolsControl = function (field) {
                    scope.currentToolsControl = field;
                };

                scope.setCurrentRemoveControl = function (Control) {
                    scope.currentRemoveControl = Control;
                };

                scope.setCurrentMoveControl = function (Control) {
                    scope.currentMoveControl = Control;
                };

                scope.setCurrentContainer = function (container) {
                    scope.currentContainer = container;
                };





                // *********************************************
                // Field conditions
                // *********************************************
                scope.addConditionRule = function (condition) {
                    if (!condition.rules){
                        condition.rules = [];
                    }

                    condition.rules.push({
                        field: condition.$newrule.field,
                        operator: condition.$newrule.operator,
                        value: condition.$newrule.value
                    });

                    condition.$newrule.field = null;
                    condition.$newrule.operator = null;
                    condition.$newrule.value = null;
                };

                scope.deleteFieldConditionRule = function (rules, rule) {
                    var index = rules.indexOf(rule);
                    rules.splice(index, 1);
                };


                // *********************************************
                // Sorting configurations
                // *********************************************
                scope.fieldSetSortableOptions = {
                        distance: 10,
                        cursor: "move",
                        placeholder: 'ui-sortable-placeholder',
                        handle: '.cell-tools-move',
                        connectWith: ".umb-forms-fieldsets"
                };

                scope.fieldSortableOptions = {
                    distance: 10,
                    cursor: "move",
                    placeholder: 'ui-sortable-placeholder',
                    handle: '.cell-tools-move',
                    connectWith: ".umb-forms-fields-container"
                };

                // *********************************************
                // Add items overlay menu
                // *********************************************
                scope.overlayMenu = {
                    show: false,
                    style: {},
                    container: undefined,
                    fieldset: undefined
                };

                scope.addItemOverlay = function(sender, field, fieldset, container, index){
                    scope.overlayMenu.container = container;
                    scope.overlayMenu.fieldset = fieldset;
                    scope.overlayMenu.index = index;
                    scope.overlayMenu.style = {};
                    scope.overlayMenu.field = field;

                    //todo calculate position...
                    var offset = $(sender.target).offset();
                    var height = $(window).height();
                    var width = $(window).width();

                    if((height-offset.top) < 250){
                        scope.overlayMenu.style.bottom = 0;
                        scope.overlayMenu.style.top = "initial";
                    }else if(offset.top < 300){
                        scope.overlayMenu.style.top = 190;
                    }

                    scope.overlayMenu.show = true;
                };

                scope.closeItemOverlay = function(){
                    scope.overlayMenu.show = false;
                    scope.overlayMenu.field = undefined;
                };


                scope.adjustSize = function(ev){
                    if(ev.target.scrollHeight > ev.target.clientHeight){
                        $(ev.target).height(ev.target.scrollHeight);
                    }
                };


                // *********************************************
                // Button functons
                // *********************************************

                scope.editForm = function (form, section) {
                    dialogService.closeAll();
                    dialogService.open(
                        {
                            template: "/app_plugins/UmbracoForms/Backoffice/Form/dialogs/formsettings.html",
                            form: form,
                            section: section,
                            page: scope.currentPage
                        });
                };

                // *********************************************
                // Internal functons
                // *********************************************
                var populateFields = function() {
                    scope.fields = [];
                    angular.forEach(scope.form.pages, function(page) {
                        angular.forEach(page.fieldSets, function(fieldset) {
                            angular.forEach(fieldset.containers, function(container) {
                                angular.forEach(container.fields, function (field) {
                                    scope.fields.push(field);
                                });
                            });
                        });
                    });
                };

                scope.initForm(scope.form, scope.fieldtypes);
            }
        };
    });
