angular.module("umbraco.directives")
    .directive('umbFormsDesignerNew', function (formService, workflowResource, notificationsService) {
        return {
            scope: {
                form: "=",
                fieldtypes: "=",
                prevaluesources: "=",
                security: "="
            },
            transclude: true,
            restrict: 'E',
            replace: true,
            templateUrl: '/App_Plugins/UmbracoForms/directives/umb-forms-designer-new.html',
            link: function (scope, element, attrs, ctrl) {


                scope.sortingMode = false;
                scope.sortingButtonKey = "general_reorder";
                scope.state = "";


                // *********************************************
                // Sorting management functions
                // *********************************************


                scope.setActive = function(fieldSet) {

	                angular.forEach(scope.form.pages, function (page) {
		                angular.forEach(page.fieldSets, function (fieldset) {

			                if (fieldset.state === "active") {
				                fieldset.state = "InActive";
			                }
			                
		                });
	                });

	                fieldSet.state = "active";
                };

                scope.sortablePages = {
                    distance: 10,
                    tolerance: "pointer",
                    opacity: 0.7,
                    scroll: true,
                    cursor: "move",
                    placeholder: "umb-forms__page-placeholder",
                    zIndex: 6000,
                    handle: ".sortable-handle",
                    items: ".sortable",
                    start: function (e, ui) {
                        ui.placeholder.height(ui.item.height());
                    }
                };

                scope.sortableFieldsets = {
                    distance: 10,
                    tolerance: "pointer",
                    connectWith: ".umb-forms__fieldsets",
                    opacity: 0.7,
                    scroll: true,
                    cursor: "move",
                    placeholder: "umb-forms__fieldset-placeholder",
                    zIndex: 6000,
                    handle: ".sortable-handle",
                    items: ".sortable",
                    start: function (e, ui) {
                        ui.placeholder.height(ui.item.height());
                    },
                    over: function (e, ui) {
                        scope.$apply(function () {
                            $(e.target).scope().page.dropOnEmpty = true;
                        });
                    },
                    out: function (e, ui) {
                        scope.$apply(function () {
                            $(e.target).scope().page.dropOnEmpty = false;
                        });
                    }
                };

                scope.sortableFields = {
                    distance: 10,
                    tolerance: "pointer",
                    connectWith: ".umb-forms__fields",
                    opacity: 0.7,
                    scroll: true,
                    cursor: "move",
                    placeholder: "umb-forms__field-placeholder",
                    zIndex: 6000,
                    handle: ".sortable-handle",
                    items: ".sortable",
                    start: function (e, ui) {
                        ui.placeholder.height(ui.item.height());
                    },
                    over: function (e, ui) {
                        scope.$apply(function () {
                            $(e.target).scope().container.dropOnEmpty = true;
                        });
                    },
                    out: function (e, ui) {
                        scope.$apply(function () {
                            $(e.target).scope().container.dropOnEmpty = false;
                        });
                    }
                };

                scope.toggleSortingMode = function () {
                    scope.sortingMode = !scope.sortingMode;

                    if (scope.sortingMode) {
                        scope.sortingButtonKey = "general_reorderDone";
                    } else {
                        scope.sortingButtonKey = "general_reorder";
                    }

                };

                // *********************************************
                // Form management functions
                // *********************************************
                scope.initForm = function (form, fieldtypes) {
                    formService.initForm(form, fieldtypes);
                };


                // *********************************************
                // Delete prompt
                // *********************************************
                scope.togglePrompt = function (object) {
                    object.deletePrompt = !object.deletePrompt;
                };

                scope.hidePrompt = function (object) {
                    object.deletePrompt = false;
                };

                // *********************************************
                // Page management functions
                // *********************************************

                scope.addPage = function (form) {
                    formService.addPage(form);
                };

                scope.removePage = function (pages, index) {
                    pages.splice(index, 1);
                };

                scope.formHasFields = function (form) {
                    var hasFields = false;

                    angular.forEach(scope.form.pages, function (page) {
                        angular.forEach(page.fieldSets, function (fieldset) {
                            angular.forEach(fieldset.containers, function (container) {
                                if (container.fields.length > 0) {
                                    hasFields = true;
                                }
                            });
                        });
                    });

                    return hasFields;
                };

                scope.pageHasFields = function (page) {

                    var hasFields = false;

                    angular.forEach(page.fieldSets, function (fieldset) {
                        angular.forEach(fieldset.containers, function (container) {
                            if (container.fields.length > 0) {
                                hasFields = true;
                            }
                        });
                    });

                    return hasFields;

                };

                // *********************************************
                // Fieldset management functions
                // *********************************************


                scope.addFieldset = function (page) {
                    // always add it last
                    var _index = page.fieldSets.length;
                    formService.addFieldset(page, _index);
                };

                scope.removeFieldset = function (page, fieldset) {
                    formService.deleteFieldset(page, fieldset);
                };

                scope.editFieldset = function (fieldset) {
                    populateFields();

                    scope.fieldsetSettingsOverlay = {
                        view: "/app_plugins/UmbracoForms/Backoffice/Form/overlays/fieldsetsettings/fieldset-settings.html",
                        title: "Edit group",
                        fieldset: fieldset,
                        fields: scope.fields,
                        closeButtonLabel: "Cancel",
                        show: true,
                        submit: function (model) {
                            scope.fieldsetSettingsOverlay.show = false;
                            scope.fieldsetSettingsOverlay = null;
                        },
                        close: function (oldModel) {
                            fieldset.containers = oldModel.fieldset.containers;
                            fieldset.condition = oldModel.fieldset.condition;

                            scope.fieldsetSettingsOverlay.show = false;
                            scope.fieldsetSettingsOverlay = null;
                        }
                    };
                };


                // *********************************************
                // Field management functions
                // *********************************************

                scope.addField = function (fieldset, container) {

                    populateFields();

                    var emptyField = formService.addEmptyField(container);

                    scope.fieldSettingsOverlay = {
                        view: "/app_plugins/UmbracoForms/Backoffice/Form/overlays/fieldsettings/field-settings.html",
                        title: "Add question",
                        field: emptyField,
                        fields: scope.fields,
                        prevalueSources: scope.prevaluesources,
                        show: true,
                        submit: function (model) {

                            emptyField.settings = {};

                            for (var i = 0; i < model.field.$fieldType.settings.length; i++) {
                                var setting = model.field.$fieldType.settings[i];
                                var key = setting.alias;
                                var value = setting.value;
                                emptyField.settings[key] = value;
                            }

                            scope.fieldSettingsOverlay.show = false;
                            scope.fieldSettingsOverlay = null;
                        },
                        close: function (oldModel) {
                            formService.deleteField(fieldset, container, emptyField);
                            scope.fieldSettingsOverlay.show = false;
                            scope.fieldSettingsOverlay = null;
                        }
                    };

                };

                scope.openFieldSettings = function (field) {

                    populateFields();

                    scope.setFieldType(field);

                    scope.fieldSettingsOverlay = {
                        view: "/app_plugins/UmbracoForms/Backoffice/Form/overlays/fieldsettings/field-settings.html",
                        title: "Edit question",
                        field: field,
                        fields: scope.fields,
                        prevalueSources: scope.prevaluesources,
                        show: true,
                        submit: function (model) {

                            field.settings = {};

                            for (var i = 0; i < model.field.$fieldType.settings.length; i++) {
                                var setting = model.field.$fieldType.settings[i];
                                var key = setting.alias;
                                var value = setting.value;
                                field.settings[key] = value;
                            }

                            scope.fieldSettingsOverlay.show = false;
                            scope.fieldSettingsOverlay = null;
                        }
                    };
                };

                scope.removeField = function (fieldset, container, field) {
                    formService.deleteField(fieldset, container, field);
                };

                scope.setFieldType = function (field) {

                    //set settings
                    angular.forEach(field.settings, function (setting) {
                        for (var key in field.settings) {
                            if (field.settings.hasOwnProperty(key)) {
                                if (_.where(field.$fieldType.settings, { alias: key }).length > 0) {
                                    _.where(field.$fieldType.settings, { alias: key })[0].value = field.settings[key];
                                }
                            }
                        }
                    });

                };

                // *********************************************
                // Field conditions
                // *********************************************

                scope.getFieldNameFromGuid = function (selectedFieldId) {
                    populateFields();
                    for (var i = 0; i < scope.fields.length; i++) {
                        var field = scope.fields[i];
                        if (field.id === selectedFieldId) {
                            return field.caption;
                        }
                    }
                };

                // *********************************************
                // Button functions
                // *********************************************

                scope.editWorkflows = function () {

                    if (scope.security && scope.security.userSecurity.manageWorkflows) {

                        populateFields();

                        scope.workflowsOverlay = {
                            view: "/app_plugins/UmbracoForms/Backoffice/Form/overlays/workflows/workflows-overview.html",
                            title: "Workflows",
                            formWorkflows: scope.form.formWorkflows,
                            messageOnSubmit: scope.form.messageOnSubmit,
                            goToPageOnSubmit: scope.form.goToPageOnSubmit,
                            submitLabel: scope.form.submitLabel,
                            manualApproval: scope.form.manualApproval,
                            fields: scope.fields,
                            closeButtonLabel: "Cancel",
                            show: true,
                            submit: function (model) {
                                scope.form.formWorkflows = model.formWorkflows;
                                scope.form.messageOnSubmit = model.messageOnSubmit;
                                scope.form.goToPageOnSubmit = model.goToPageOnSubmit;

                                scope.workflowsOverlay.show = false;
                                scope.workflowsOverlay = null;
                            },
                            close: function (oldModel) {
                                // reset the model
                                scope.form.formWorkflows = oldModel.formWorkflows;
                                scope.form.messageOnSubmit = oldModel.messageOnSubmit;
                                scope.form.goToPageOnSubmit = oldModel.goToPageOnSubmit;

                                scope.workflowsOverlay.show = false;
                                scope.workflowsOverlay = null;
                            }
                        };

                    }
                };

                scope.editWorkflowSettings = function (workflow) {

                    if (scope.security && scope.security.userSecurity.manageWorkflows) {

                        populateFields();

                        scope.workflowSettingsOverlay = {
                            view: "/app_plugins/UmbracoForms/Backoffice/Form/overlays/workflows/workflow-settings.html",
                            workflow: workflow,
                            fields: scope.fields,
                            title: workflow.name,
                            show: true,
                            submit: function (model) {

                                //Validate settings
                                workflowResource.validateWorkflowSettings(model.workflow).then(function (response) {
                                    if (response.data.length > 0) {
                                        angular.forEach(response.data, function (error) {
                                            notificationsService.error("Workflow failed to save", error.Message);
                                        });
                                    } else {
                                        scope.workflowSettingsOverlay.show = false;
                                        scope.workflowSettingsOverlay = null;
                                    }

                                });
                            }
                        };

                    }

                };

                scope.editSubmitMessageWorkflow = function () {

                    scope.submitMessageWorkflowOverlay = {
                        view: "/app_plugins/UmbracoForms/Backoffice/Form/overlays/workflows/submit-message-workflow-settings.html",
                        title: "Message on submit",
                        messageOnSubmit: scope.form.messageOnSubmit,
                        goToPageOnSubmit: scope.form.goToPageOnSubmit,
                        show: true,
                        submit: function (model) {
                            scope.form.messageOnSubmit = model.messageOnSubmit;
                            scope.form.goToPageOnSubmit = model.goToPageOnSubmit;

                            scope.submitMessageWorkflowOverlay.show = false;
                            scope.submitMessageWorkflowOverlay = null;
                        }
                    };

                };

                // *********************************************
                // Internal functions
                // *********************************************
                var populateFields = function () {
                    scope.fields = [];
                    angular.forEach(scope.form.pages, function (page) {
                        angular.forEach(page.fieldSets, function (fieldset) {
                            angular.forEach(fieldset.containers, function (container) {
                                angular.forEach(container.fields, function (field) {
                                    scope.fields.push(field);
                                });
                            });
                        });
                    });
                };

                scope.initForm(scope.form, scope.fieldtypes);
            }
        };
    });
