function formService(preValueSourceResource) {

	var generateGUID = function() {
	    var d = new Date().getTime();

	    var uuid = 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
	        var r = (d + Math.random() * 16) % 16 | 0;
	        d = Math.floor(d / 16);
	        return (c === 'x' ? r : (r & 0x7 | 0x8)).toString(16);
	    });

	    return uuid;
	};


	var service = {
		fieldTypes: [],
		actionTypes: [
			{
				name: "show",
				value: "Show"
			},
			{
				name: "hide",
				value: "Hide"
			}
		],
		logicTypes: [
			{
				name: "all",
				value: "All"
			},
			{
				name: "any",
				value: "Any"
			}
		],
		operators: [
			{
				name: "is",
				value: "Is"
			},
			{
				name: "is not",
				value: "IsNot"
			},
			{
				name: "is greater than",
				value: "GreaterThen"
			},
			{
				name: "is less than",
				value: "LessThen"
			},
			{
				name: "contains",
				value: "Contains"
			},
			{
				name: "starts with",
				value: "StartsWith"
			},
			{
				name: "ends with",
				value: "EndsWith"
			}
		],

		getActionTypes: function() {
			return service.actionTypes;
		},

		getLogicTypes: function() {
			return service.logicTypes;
		},

		getOperators: function() {
			return service.operators;
		},

		initForm: function(form, fieldtypes){
			service.fieldTypes = fieldtypes;

			if(!form.pages || form.pages.length === 0){
			    service.addPage(form);
			}else{

				_.each(service.getAllFields(form), function(field){

					if(!field.$fieldType){
						service.setFieldType(field,field.fieldTypeId);
					}

				});
			}
		},

		addPage: function(form, index){
			var p = {caption: "", fieldSets: []};
			service.addFieldset(p);

			if(form.pages.length > index){
				form.pages.splice(index, 0, p);
			}else{
				form.pages.push(p);
			}
		},

		addFieldset: function(page, index){
			var fs = {caption: "", containers: [], id:generateGUID()};
			service.addContainer(fs);

			if(page.fieldSets.length > index){
				page.fieldSets.splice(index, 0, fs);
			}else{
				page.fieldSets.push(fs);
			}
		},

		deleteFieldset : function(page, fieldset){
			if(page.fieldSets.length > 1){
				var index = page.fieldSets.indexOf(fieldset);
				page.fieldSets.splice(index, 1);
			}else{
				fieldset.containers.length = 0;
				service.addContainer(fieldset);
			}
		},

		deleteFieldsetAtIndex : function(page, index){
			if(page.fieldSets.length > 1){
				page.fieldSets.splice(index, 1);
			}else{
				fieldset.containers.length = 0;
				service.addContainer(fieldset);
			}
		},

		splitFieldset: function(page, fieldset, container, splitAtIndex){

			var newfieldset = {caption: "", containers: [{ caption: "", fields: [] }] };
			var insertAt = page.fieldSets.indexOf(fieldset);

			page.fieldSets.splice(insertAt+1, 0, newfieldset);

			var oldFields = container.fields.slice(0,splitAtIndex+1);
			var newFields = container.fields.slice(splitAtIndex+1);

			newfieldset.containers[0].fields = newFields;
			container.fields = oldFields;
		},

		addContainer: function(fieldset, index){
			var c = { caption: "", fields: [] };

			if(fieldset.containers.length > index){
				fieldset.containers.splice(index, 0, c);
			}else{
				fieldset.containers.push(c);
			}
		},

		splitContainer: function(fieldset, container, splitAtIndex){

			var newContainer = { caption: "", fields: [] };
			var insertAt = fieldset.containers.indexOf(container);

			fieldset.containers.splice(insertAt-1, 0, newContainer);
			var newFields = container.fields.slice(0,splitAtIndex+1);
			var oldFields = container.fields.slice(splitAtIndex+1);

			newContainer.fields = newFields;
			container.fields = oldFields;
		},

		deleteContainer: function(fieldset, container){
			//only delete the container if there are multiple ones on this fieldseet
			//otherwise keep it and just clear its contents
			if(fieldset.containers.length > 1){
				var index = fieldset.containers.indexOf(container);
				if(index >= 0){
					service.deleteContainerAtIndex(fieldset, index);
				}
			}else{
				container.fields.length = 0;
			}
		},

		deleteContainerAtIndex: function(fieldset, index){

			if(fieldset.containers.length > 1){
				fieldset.containers.splice(index, 1);
			}else{
				fieldset.containers.length = 0;
			}
		},


		syncContainerWidths : function(form){
			_.each(form.pages, function(page){
				_.each(page.fieldSets, function(fieldset){
					var containers = fieldset.containers.length;
					var avrg = Math.floor(12 / containers);
					_.each(fieldset.containers, function(container){
						container.width = avrg;
					});
				});
			});
		},

		addField : function(container, fieldtype, index) {
		    var newField = {
		        caption: "",
		        id: generateGUID(),
		        settings: {},
		        preValues: [],
		        $focus: true
			};

			service.loadFieldTypeSettings(newField, fieldtype);

			if(container.fields.length > index){
				container.fields.splice(index, 0, newField);
			}else{
				container.fields.push(newField);
			}

		},

		addEmptyField : function(container) {

			var newField = {
				caption: "",
				alias:"",
				id: generateGUID(),
				settings: {},
				preValues: [],
				$focus: true
			};

			container.fields.push(newField);

			return newField;

		},

		getAllFields : function(form){
			var fields = [];
			if(form.pages){
				_.each(form.pages, function(page){
					if(page.fieldSets){
						_.each(page.fieldSets, function(fieldset){
							if(fieldset.containers){
								_.each(fieldset.containers, function(container){
									if(container.fields){
										_.each(container.fields, function(field){
											fields.push(field);
										});
									}
								});
							}
						});
					}
				});
			}

			return fields;
		},


		deleteField: function(fieldset,container,field){
			var index = container.fields.indexOf(field);
			if(index >= 0){
				service.deleteFieldAtIndex(fieldset, container, index);
			}
		},


		deleteFieldAtIndex: function(fieldset,container,index){

			container.fields.splice(index, 1);

			if(container.fields.length === 0){
				service.deleteContainer(fieldset,container);
			}

		},


		setFieldType: function(field, fieldTypeId){
			//get field type
			field.fieldTypeId = fieldTypeId;

			var fldt = _.find(service.fieldTypes, function(ft){return ft.id === field.fieldTypeId; });
			field.$fieldType = fldt;

			service.loadFieldTypeSettings(field, field.$fieldType);


			service.loadFieldTypePrevalues(field);

		},

        loadFieldTypePrevalues : function(field) {

            if (field.prevalueSourceId !== null && field.prevalueSourceId !== "00000000-0000-0000-0000-000000000000") {

                preValueSourceResource.getPreValuesByGuid(field.prevalueSourceId)
                    .then(function(response) {
                        field.$preValues = response.data;

                    });
            } else {
                field.$preValues = null;
            }

        },

		loadFieldTypeSettings : function(field, fieldtype){

			var stng = angular.copy(fieldtype.settings);

			if(field.fieldTypeId !== fieldtype.id){
				field.settings = {};
			}

			field.fieldTypeId = fieldtype.id;
			field.$fieldType = fieldtype;

			if(fieldtype.settings){
				_.each(fieldtype.settings, function(setting){
					if(!field.settings[setting.alias]){
						field.settings[setting.alias] = "";
					}
				});
			}
		},


		deleteConditionRule : function(rules, rule) {
		    var index = rules.indexOf(rule);
		    rules.splice(index, 1);
		},

        addConditionRule : function(condition) {
	        if (!condition.rules) {
	            condition.rules = [];
	        }

	        condition.rules.push({
	            field: condition.$newrule.field,
	            operator: condition.$newrule.operator,
	            value: condition.$newrule.value
	        });
        },

		addEmptyConditionRule : function(condition) {
			if (!condition.rules) {
	            condition.rules = [];
	        }

			condition.rules.push({
				field: "",
				operator: "",
				value: ""
			});
		},

		populateConditionRulePrevalues: function(selectedField, rule, fields) {

			for(var i = 0; i < fields.length; i++) {
				var field = fields[i];

				if(field.id === selectedField) {

					// prevalues and be stored in both $preValues and preValues
					if(field.$preValues && field.$preValues.length > 0) {

						rule.$preValues = field.$preValues;

					} else if(field.preValues && field.preValues.length > 0) {

						var rulePreValuesObjectArray = [];

						// make prevalues to object array
						for(var preValueIndex = 0; preValueIndex < field.preValues.length; preValueIndex++) {

							var preValue = field.preValues[preValueIndex];
							var preValueObject = {
								value: preValue
							};

							rulePreValuesObjectArray.push(preValueObject);
						}

						rule.$preValues = rulePreValuesObjectArray;

					} else {
						rule.$preValues = null;
					}

				}
			}

		}

	};

	return service;
}
angular.module('umbraco.services').factory('formService', formService);
